@@ -22,7 +22,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -170,6 +170,59 @@ skip_space (const char **pos)
 	return !(*p == '\0');
 }
   
+#ifdef G_OS_WIN32
+
+/* DllMain function needed to tuck away the gdk-pixbuf DLL name */
+G_WIN32_DLLMAIN_FOR_DLL_NAME (static, dll_name)
+
+static char *
+get_toplevel (void)
+{
+  static char *toplevel = NULL;
+
+  if (toplevel == NULL)
+    toplevel = g_win32_get_package_installation_subdirectory
+      (GETTEXT_PACKAGE, dll_name, "");
+
+  return toplevel;
+}
+
+static char *
+get_sysconfdir (void)
+{
+  static char *sysconfdir = NULL;
+
+  if (sysconfdir == NULL)
+    sysconfdir = g_win32_get_package_installation_subdirectory
+      (GETTEXT_PACKAGE, dll_name, "etc");
+
+  return sysconfdir;
+}
+
+#undef GTK_SYSCONFDIR
+#define GTK_SYSCONFDIR get_sysconfdir()
+
+static void
+correct_prefix (gchar **path)
+{
+  if (strncmp (*path, GTK_PREFIX "/", strlen (GTK_PREFIX "/")) == 0 ||
+      strncmp (*path, GTK_PREFIX "\\", strlen (GTK_PREFIX "\\")) == 0)
+    {
+      /* This is an entry put there by gdk-pixbuf-query-loaders on the
+       * packager's system. On Windows a prebuilt GTK+ package can be
+       * installed in a random location. The gdk-pixbuf.loaders file
+       * distributed in such a package contains paths from the package
+       * builder's machine. Replace the build-time prefix with the
+       * installation prefix on this machine.
+       */
+      gchar *tem = *path;
+      *path = g_strconcat (get_toplevel (), tem + strlen (GTK_PREFIX), NULL);
+      g_free (tem);
+    }
+}
+
+#endif
+
 static gchar *
 gdk_pixbuf_get_module_file (void)
 {
@@ -214,6 +267,9 @@ gdk_pixbuf_io_init ()
 				/* Blank line marking the end of a module
 				 */
 			if (module && *p != '#') {
+#ifdef G_OS_WIN32
+				correct_prefix (&module->module_path);
+#endif
 				file_formats = g_slist_prepend (file_formats, module);
 				module = NULL;
 			}
@@ -336,29 +392,6 @@ gdk_pixbuf_io_init ()
 	g_free (filename);
 }
 
-#ifdef G_OS_WIN32
-
-/* DllMain function needed to tuck away the gdk-pixbuf DLL name */
-
-G_WIN32_DLLMAIN_FOR_DLL_NAME (static, dll_name)
-
-static char *
-get_libdir (void)
-{
-  static char *libdir = NULL;
-
-  if (libdir == NULL)
-    libdir = g_win32_get_package_installation_subdirectory
-      (GETTEXT_PACKAGE, dll_name, "lib\\gtk-2.0\\" GTK_BINARY_VERSION "\\loaders");
-
-  return libdir;
-}
-
-#undef PIXBUF_LIBDIR
-#define PIXBUF_LIBDIR get_libdir ()
-
-#endif
-
 /* actually load the image handler - gdk_pixbuf_get_module only get a */
 /* reference to the module to load, it doesn't actually load it       */
 /* perhaps these actions should be combined in one function           */
@@ -884,7 +917,7 @@ gdk_pixbuf_new_from_file_at_size (const char *filename,
 			if (!gdk_pixbuf_loader_write (loader, buffer, length, error)) {
 				gdk_pixbuf_loader_close (loader, NULL);
 				fclose (f);
-				g_object_unref (G_OBJECT (loader));
+				g_object_unref (loader);
 				return NULL;
 			}
 	}
@@ -892,14 +925,14 @@ gdk_pixbuf_new_from_file_at_size (const char *filename,
 	fclose (f);
 
 	if (!gdk_pixbuf_loader_close (loader, error)) {
-		g_object_unref (G_OBJECT (loader));
+		g_object_unref (loader);
 		return NULL;
 	}
 
 	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 
 	if (!pixbuf) {
-		g_object_unref (G_OBJECT (loader));
+		g_object_unref (loader);
 		g_set_error (error,
                              GDK_PIXBUF_ERROR,
                              GDK_PIXBUF_ERROR_FAILED,
@@ -910,7 +943,7 @@ gdk_pixbuf_new_from_file_at_size (const char *filename,
 
 	g_object_ref (pixbuf);
 
-	g_object_unref (G_OBJECT (loader));
+	g_object_unref (loader);
 
 	return pixbuf;
 }
@@ -956,7 +989,6 @@ gdk_pixbuf_get_file_info (const gchar  *filename,
 			  gint         *height)
 {
 	GdkPixbufLoader *loader;
-	GError *temp = NULL;
 	guchar buffer [4096];
 	int length;
 	FILE *f;
@@ -992,7 +1024,7 @@ gdk_pixbuf_get_file_info (const gchar  *filename,
 
 	fclose (f);
 	gdk_pixbuf_loader_close (loader, NULL);
-	g_object_unref (G_OBJECT (loader));
+	g_object_unref (loader);
 
 	if (width) 
 		*width = info.width;
