@@ -39,7 +39,7 @@
  * not have TrackMouseEvent at all (?) --hb
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <glib/gprintf.h>
 
@@ -79,6 +79,11 @@
 #define XBUTTON2 2
 #endif
 
+#ifndef VK_XBUTTON1
+#define VK_XBUTTON1 5
+#define VK_XBUTTON2 6
+#endif
+
 #ifndef MK_XBUTTON1
 #define MK_XBUTTON1 32
 #define MK_XBUTTON2 64
@@ -560,6 +565,7 @@ gdk_pointer_grab (GdkWindow    *window,
   
   if (return_val == GDK_GRAB_SUCCESS)
     {
+      GdkWindowImplWin32 *impl = GDK_WINDOW_IMPL_WIN32 (((GdkWindowObject *) window)->impl);
       p_grab_window = window;
 
       if (p_grab_cursor != NULL)
@@ -573,6 +579,10 @@ gdk_pointer_grab (GdkWindow    *window,
 
       if (p_grab_cursor != NULL)
 	SetCursor (p_grab_cursor);
+      else if (impl->hcursor != NULL)
+	SetCursor (impl->hcursor);
+      else
+	SetCursor (LoadCursor (NULL, IDC_ARROW));
 
       if (confine_to != NULL)
 	{
@@ -807,7 +817,7 @@ gdk_add_client_message_filter (GdkAtom       message_type,
   filter->function = func;
   filter->data = data;
   
-  client_filters = g_list_prepend (client_filters, filter);
+  client_filters = g_list_append (client_filters, filter);
 }
 
 static void
@@ -822,6 +832,17 @@ build_key_event_state (GdkEvent *event,
   if (key_state[VK_CAPITAL] & 0x01)
     event->key.state |= GDK_LOCK_MASK;
 
+  if (key_state[VK_LBUTTON] & 0x80)
+    event->key.state |= GDK_BUTTON1_MASK;
+  if (key_state[VK_MBUTTON] & 0x80)
+    event->key.state |= GDK_BUTTON2_MASK;
+  if (key_state[VK_RBUTTON] & 0x80)
+    event->key.state |= GDK_BUTTON3_MASK;
+  if (key_state[VK_XBUTTON1] & 0x80)
+    event->key.state |= GDK_BUTTON4_MASK;
+  if (key_state[VK_XBUTTON2] & 0x80)
+    event->key.state |= GDK_BUTTON5_MASK;
+
   /* Win9x doesn't distinguish between left and right Control and Alt
    * in the keyboard state as returned by GetKeyboardState(), so we
    * have to punt, and accept either Control + either Alt to be AltGr.
@@ -1264,6 +1285,9 @@ synthesize_leave_event (GdkWindow      *window,
 {
   POINT pt;
 
+  if (p_grab_window != NULL && !p_grab_owner_events && !(p_grab_mask & GDK_LEAVE_NOTIFY_MASK))
+    return;
+
   if (!(((GdkWindowObject *) window)->event_mask & GDK_LEAVE_NOTIFY_MASK))
     return;
 
@@ -1295,6 +1319,9 @@ synthesize_enter_event (GdkWindow      *window,
 {
   POINT pt;
 
+  if (p_grab_window != NULL && !p_grab_owner_events && !(p_grab_mask & GDK_ENTER_NOTIFY_MASK))
+    return;
+
   if (!(((GdkWindowObject *) window)->event_mask & GDK_ENTER_NOTIFY_MASK))
     return;
 
@@ -2793,6 +2820,9 @@ gdk_event_translate (GdkDisplay *display,
 
     case WM_SETFOCUS:
     case WM_KILLFOCUS:
+      if (p_grab_window != NULL && !p_grab_owner_events && !(p_grab_mask & GDK_FOCUS_CHANGE_MASK))
+	break;
+
       if (!(((GdkWindowObject *) window)->event_mask & GDK_FOCUS_CHANGE_MASK))
 	break;
 
@@ -3163,14 +3193,19 @@ gdk_event_translate (GdkDisplay *display,
 
       if (impl->hint_flags & GDK_HINT_MAX_SIZE)
 	{
+	  int maxw, maxh;
+
 	  rect.left = rect.top = 0;
 	  rect.right = impl->hints.max_width;
 	  rect.bottom = impl->hints.max_height;
 
 	  _gdk_win32_adjust_client_rect (window, &rect);
 
-	  mmi->ptMaxTrackSize.x = rect.right - rect.left;
-	  mmi->ptMaxTrackSize.y = rect.bottom - rect.top;
+	  /* at least on win9x we have the 16 bit trouble */
+	  maxw = rect.right - rect.left;
+	  maxh = rect.bottom - rect.top;
+	  mmi->ptMaxTrackSize.x = maxw > 0 && maxw < G_MAXSHORT ? maxw : G_MAXSHORT;
+	  mmi->ptMaxTrackSize.y = maxh > 0 && maxh < G_MAXSHORT ? maxw : G_MAXSHORT;
 	}
 
       if (impl->hint_flags & (GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE))
@@ -3427,3 +3462,9 @@ gdk_display_flush (GdkDisplay * display)
 
   /* Nothing */
 }
+
+gboolean
+gdk_net_wm_supports (GdkAtom property)
+{
+  return FALSE;
+}
