@@ -17,11 +17,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <stdlib.h>
-#include <gtk/gtkcellrenderertoggle.h>
-#include <gtk/gtksignal.h>
+#include "gtkcellrenderertoggle.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
+#include "gtktreeprivate.h"
 
 static void gtk_cell_renderer_toggle_get_property  (GObject                    *object,
 						    guint                       param_id,
@@ -65,19 +66,27 @@ enum {
   PROP_ZERO,
   PROP_ACTIVATABLE,
   PROP_ACTIVE,
-  PROP_RADIO
+  PROP_RADIO,
+  PROP_INCONSISTENT
 };
 
-
 #define TOGGLE_WIDTH 12
 
 static guint toggle_cell_signals[LAST_SIGNAL] = { 0 };
 
+#define GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_CELL_RENDERER_TOGGLE, GtkCellRendererTogglePrivate))
 
-GtkType
+typedef struct _GtkCellRendererTogglePrivate GtkCellRendererTogglePrivate;
+struct _GtkCellRendererTogglePrivate
+{
+  guint inconsistent : 1;
+};
+
+
+GType
 gtk_cell_renderer_toggle_get_type (void)
 {
-  static GtkType cell_toggle_type = 0;
+  static GType cell_toggle_type = 0;
 
   if (!cell_toggle_type)
     {
@@ -94,7 +103,9 @@ gtk_cell_renderer_toggle_get_type (void)
 	(GInstanceInitFunc) gtk_cell_renderer_toggle_init,
       };
 
-      cell_toggle_type = g_type_register_static (GTK_TYPE_CELL_RENDERER, "GtkCellRendererToggle", &cell_toggle_info, 0);
+      cell_toggle_type =
+	g_type_register_static (GTK_TYPE_CELL_RENDERER, "GtkCellRendererToggle",
+				&cell_toggle_info, 0);
     }
 
   return cell_toggle_type;
@@ -127,8 +138,17 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
   g_object_class_install_property (object_class,
 				   PROP_ACTIVE,
 				   g_param_spec_boolean ("active",
-							 _("Toggle state"),
-							 _("The toggle state of the button"),
+							 P_("Toggle state"),
+							 P_("The toggle state of the button"),
+							 FALSE,
+							 G_PARAM_READABLE |
+							 G_PARAM_WRITABLE));
+
+  g_object_class_install_property (object_class,
+		                   PROP_INCONSISTENT,
+				   g_param_spec_boolean ("inconsistent",
+					                 P_("Inconsistent state"),
+							 P_("The inconsistent state of the button"),
 							 FALSE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -136,8 +156,8 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
   g_object_class_install_property (object_class,
 				   PROP_ACTIVATABLE,
 				   g_param_spec_boolean ("activatable",
-							 _("Activatable"),
-							 _("The toggle button can be activated"),
+							 P_("Activatable"),
+							 P_("The toggle button can be activated"),
 							 TRUE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -145,21 +165,24 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
   g_object_class_install_property (object_class,
 				   PROP_RADIO,
 				   g_param_spec_boolean ("radio",
-							 _("Radio state"),
-							 _("Draw the toggle button as a radio button"),
+							 P_("Radio state"),
+							 P_("Draw the toggle button as a radio button"),
 							 FALSE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
 
 
   toggle_cell_signals[TOGGLED] =
-    gtk_signal_new ("toggled",
-		    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkCellRendererToggleClass, toggled),
-		    _gtk_marshal_VOID__STRING,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_STRING);
+    g_signal_new ("toggled",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GtkCellRendererToggleClass, toggled),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__STRING,
+		  G_TYPE_NONE, 1,
+		  G_TYPE_STRING);
+
+  g_type_class_add_private (object_class, sizeof (GtkCellRendererTogglePrivate));
 }
 
 static void
@@ -169,12 +192,18 @@ gtk_cell_renderer_toggle_get_property (GObject     *object,
 				       GParamSpec  *pspec)
 {
   GtkCellRendererToggle *celltoggle = GTK_CELL_RENDERER_TOGGLE (object);
+  GtkCellRendererTogglePrivate *priv;
+
+  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (object);
   
   switch (param_id)
     {
     case PROP_ACTIVE:
       g_value_set_boolean (value, celltoggle->active);
       break;
+    case PROP_INCONSISTENT:
+      g_value_set_boolean (value, priv->inconsistent);
+      break;
     case PROP_ACTIVATABLE:
       g_value_set_boolean (value, celltoggle->activatable);
       break;
@@ -195,6 +224,9 @@ gtk_cell_renderer_toggle_set_property (GObject      *object,
 				       GParamSpec   *pspec)
 {
   GtkCellRendererToggle *celltoggle = GTK_CELL_RENDERER_TOGGLE (object);
+  GtkCellRendererTogglePrivate *priv;
+
+  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (object);
   
   switch (param_id)
     {
@@ -202,6 +234,10 @@ gtk_cell_renderer_toggle_set_property (GObject      *object,
       celltoggle->active = g_value_get_boolean (value);
       g_object_notify (G_OBJECT(object), "active");
       break;
+    case PROP_INCONSISTENT:
+      priv->inconsistent = g_value_get_boolean (value);
+      g_object_notify (G_OBJECT (object), "inconsistent");
+      break;
     case PROP_ACTIVATABLE:
       celltoggle->activatable = g_value_get_boolean (value);
       g_object_notify (G_OBJECT(object), "activatable");
@@ -232,7 +268,7 @@ gtk_cell_renderer_toggle_set_property (GObject      *object,
 GtkCellRenderer *
 gtk_cell_renderer_toggle_new (void)
 {
-  return GTK_CELL_RENDERER (gtk_type_new (gtk_cell_renderer_toggle_get_type ()));
+  return g_object_new (GTK_TYPE_CELL_RENDERER_TOGGLE, NULL);
 }
 
 static void
@@ -260,7 +296,8 @@ gtk_cell_renderer_toggle_get_size (GtkCellRenderer *cell,
     {
       if (x_offset)
 	{
-	  *x_offset = cell->xalign * (cell_area->width - calc_width);
+	  *x_offset = ((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) ?
+		       (1.0 - cell->xalign) : cell->xalign) * (cell_area->width - calc_width);
 	  *x_offset = MAX (*x_offset, 0);
 	}
       if (y_offset)
@@ -273,7 +310,7 @@ gtk_cell_renderer_toggle_get_size (GtkCellRenderer *cell,
 
 static void
 gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
-				 GdkWindow            *window,
+				 GdkDrawable          *window,
 				 GtkWidget            *widget,
 				 GdkRectangle         *background_area,
 				 GdkRectangle         *cell_area,
@@ -281,11 +318,14 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
 				 GtkCellRendererState  flags)
 {
   GtkCellRendererToggle *celltoggle = (GtkCellRendererToggle *) cell;
+  GtkCellRendererTogglePrivate *priv;
   gint width, height;
   gint x_offset, y_offset;
   GtkShadowType shadow;
   GtkStateType state = 0;
-  
+
+  priv = GTK_CELL_RENDERER_TOGGLE_GET_PRIVATE (cell);
+
   gtk_cell_renderer_toggle_get_size (cell, widget, cell_area,
 				     &x_offset, &y_offset,
 				     &width, &height);
@@ -295,7 +335,10 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
   if (width <= 0 || height <= 0)
     return;
 
-  shadow = celltoggle->active ? GTK_SHADOW_IN : GTK_SHADOW_OUT;
+  if (priv->inconsistent)
+    shadow = GTK_SHADOW_ETCHED_IN;
+  else
+    shadow = celltoggle->active ? GTK_SHADOW_IN : GTK_SHADOW_OUT;
 
   if ((flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED)
     {
@@ -317,7 +360,7 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
       gtk_paint_option (widget->style,
                         window,
                         state, shadow,
-                        cell_area, widget, "cellradio",
+                        expose_area, widget, "cellradio",
                         cell_area->x + x_offset + cell->xpad,
                         cell_area->y + y_offset + cell->ypad,
                         width - 1, height - 1);
@@ -327,7 +370,7 @@ gtk_cell_renderer_toggle_render (GtkCellRenderer      *cell,
       gtk_paint_check (widget->style,
                        window,
                        state, shadow,
-                       cell_area, widget, "cellcheck",
+                       expose_area, widget, "cellcheck",
                        cell_area->x + x_offset + cell->xpad,
                        cell_area->y + y_offset + cell->ypad,
                        width - 1, height - 1);
@@ -348,7 +391,7 @@ gtk_cell_renderer_toggle_activate (GtkCellRenderer      *cell,
   celltoggle = GTK_CELL_RENDERER_TOGGLE (cell);
   if (celltoggle->activatable)
     {
-      gtk_signal_emit (GTK_OBJECT (cell), toggle_cell_signals[TOGGLED], path);
+      g_signal_emit (cell, toggle_cell_signals[TOGGLED], 0, path);
       return TRUE;
     }
 
@@ -423,5 +466,5 @@ gtk_cell_renderer_toggle_set_active (GtkCellRendererToggle *toggle,
 {
   g_return_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (toggle));
 
-  g_object_set (G_OBJECT (toggle), "active", setting?TRUE:FALSE, NULL);
+  g_object_set (toggle, "active", setting ? TRUE : FALSE, NULL);
 }
