@@ -19,6 +19,7 @@
  * Global clipboard abstraction. 
  */
 
+#include <config.h>
 #include <string.h>
 
 #include "gtkclipboard.h"
@@ -143,6 +144,8 @@ static void
 gtk_clipboard_finalize   (GObject *object)
 {
   clipboard_unset (GTK_CLIPBOARD (object));
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
