@@ -24,6 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <math.h>
 #include "gtkcontainer.h"
 #include "gtkimage.h"
@@ -45,6 +46,8 @@ static void gtk_image_size_request (GtkWidget      *widget,
 static void gtk_image_destroy      (GtkObject      *object);
 static void gtk_image_clear        (GtkImage       *image);
 static void gtk_image_reset        (GtkImage       *image);
+static void gtk_image_calc_size    (GtkImage       *image);
+
 static void gtk_image_update_size  (GtkImage       *image,
                                     gint            image_width,
                                     gint            image_height);
@@ -130,40 +133,40 @@ gtk_image_class_init (GtkImageClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_PIXBUF,
                                    g_param_spec_object ("pixbuf",
-                                                        _("Pixbuf"),
-                                                        _("A GdkPixbuf to display"),
+                                                        P_("Pixbuf"),
+                                                        P_("A GdkPixbuf to display"),
                                                         GDK_TYPE_PIXBUF,
                                                         G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_PIXMAP,
                                    g_param_spec_object ("pixmap",
-                                                        _("Pixmap"),
-                                                        _("A GdkPixmap to display"),
+                                                        P_("Pixmap"),
+                                                        P_("A GdkPixmap to display"),
                                                         GDK_TYPE_PIXMAP,
                                                         G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_IMAGE,
                                    g_param_spec_object ("image",
-                                                        _("Image"),
-                                                        _("A GdkImage to display"),
+                                                        P_("Image"),
+                                                        P_("A GdkImage to display"),
                                                         GDK_TYPE_IMAGE,
                                                         G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_MASK,
                                    g_param_spec_object ("mask",
-                                                        _("Mask"),
-                                                        _("Mask bitmap to use with GdkImage or GdkPixmap"),
+                                                        P_("Mask"),
+                                                        P_("Mask bitmap to use with GdkImage or GdkPixmap"),
                                                         GDK_TYPE_PIXMAP,
                                                         G_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_FILE,
                                    g_param_spec_string ("file",
-                                                        _("Filename"),
-                                                        _("Filename to load and display"),
+                                                        P_("Filename"),
+                                                        P_("Filename to load and display"),
                                                         NULL,
                                                         G_PARAM_WRITABLE));
   
@@ -171,24 +174,24 @@ gtk_image_class_init (GtkImageClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_STOCK,
                                    g_param_spec_string ("stock",
-                                                        _("Stock ID"),
-                                                        _("Stock ID for a stock image to display"),
+                                                        P_("Stock ID"),
+                                                        P_("Stock ID for a stock image to display"),
                                                         NULL,
                                                         G_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_ICON_SET,
                                    g_param_spec_boxed ("icon_set",
-                                                       _("Icon set"),
-                                                       _("Icon set to display"),
+                                                       P_("Icon set"),
+                                                       P_("Icon set to display"),
                                                        GTK_TYPE_ICON_SET,
                                                        G_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_ICON_SIZE,
                                    g_param_spec_int ("icon_size",
-                                                     _("Icon size"),
-                                                     _("Size to use for stock icon or icon set"),
+                                                     P_("Icon size"),
+                                                     P_("Size to use for stock icon or icon set"),
                                                      0, G_MAXINT,
                                                      DEFAULT_ICON_SIZE,
                                                      G_PARAM_READWRITE));
@@ -196,16 +199,16 @@ gtk_image_class_init (GtkImageClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_PIXBUF_ANIMATION,
                                    g_param_spec_object ("pixbuf_animation",
-                                                        _("Animation"),
-                                                        _("GdkPixbufAnimation to display"),
+                                                        P_("Animation"),
+                                                        P_("GdkPixbufAnimation to display"),
                                                         GDK_TYPE_PIXBUF_ANIMATION,
                                                         G_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_STORAGE_TYPE,
                                    g_param_spec_enum ("storage_type",
-                                                      _("Storage type"),
-                                                      _("The representation being used for image data"),
+                                                      P_("Storage type"),
+                                                      P_("The representation being used for image data"),
                                                       GTK_TYPE_IMAGE_TYPE,
                                                       GTK_IMAGE_EMPTY,
                                                       G_PARAM_READABLE));
@@ -1227,6 +1230,14 @@ gtk_image_expose (GtkWidget      *widget,
       misc = GTK_MISC (widget);
 
       area = event->area;
+
+      /* For stock items and icon sets, we lazily calculate
+       * the size; we might get here between a queue_resize()
+       * and size_request() if something explicitely forces
+       * a redraw.
+       */
+      if (widget->requisition.width == 0 && widget->requisition.height == 0)
+	gtk_image_calc_size (image);
       
       if (!gdk_rectangle_intersect (&area, &widget->allocation, &area))
 	return FALSE;
@@ -1591,29 +1602,25 @@ gtk_image_reset (GtkImage *image)
 }
 
 static void
-gtk_image_size_request (GtkWidget      *widget,
-                        GtkRequisition *requisition)
+gtk_image_calc_size (GtkImage *image)
 {
-  GtkImage *image;
+  GtkWidget *widget = GTK_WIDGET (image);
   GdkPixbuf *pixbuf = NULL;
   
-  image = GTK_IMAGE (widget);
-
   /* We update stock/icon set on every size request, because
    * the theme could have affected the size; for other kinds of
    * image, we just update the requisition when the image data
    * is set.
    */
-  
   switch (image->storage_type)
     {
     case GTK_IMAGE_STOCK:
-      pixbuf = gtk_widget_render_icon (GTK_WIDGET (image),
+      pixbuf = gtk_widget_render_icon (widget,
                                        image->data.stock.stock_id,
                                        image->icon_size,
                                        NULL);
       break;
-
+      
     case GTK_IMAGE_ICON_SET:
       pixbuf = gtk_icon_set_render_icon (image->data.icon_set.icon_set,
                                          widget->style,
@@ -1630,11 +1637,22 @@ gtk_image_size_request (GtkWidget      *widget,
 
   if (pixbuf)
     {
-      GTK_WIDGET (image)->requisition.width = gdk_pixbuf_get_width (pixbuf) + GTK_MISC (image)->xpad * 2;
-      GTK_WIDGET (image)->requisition.height = gdk_pixbuf_get_height (pixbuf) + GTK_MISC (image)->ypad * 2;
+      widget->requisition.width = gdk_pixbuf_get_width (pixbuf) + GTK_MISC (image)->xpad * 2;
+      widget->requisition.height = gdk_pixbuf_get_height (pixbuf) + GTK_MISC (image)->ypad * 2;
 
       g_object_unref (pixbuf);
     }
+}
+
+static void
+gtk_image_size_request (GtkWidget      *widget,
+                        GtkRequisition *requisition)
+{
+  GtkImage *image;
+  
+  image = GTK_IMAGE (widget);
+
+  gtk_image_calc_size (image);
 
   /* Chain up to default that simply reads current requisition */
   GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
@@ -1651,5 +1669,3 @@ gtk_image_update_size (GtkImage *image,
   if (GTK_WIDGET_VISIBLE (image))
     gtk_widget_queue_resize (GTK_WIDGET (image));
 }
-
-
