@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include "gtktreeselection.h"
 #include "gtktreeprivate.h"
 #include "gtkrbtree.h"
@@ -474,7 +475,7 @@ gtk_tree_selection_get_selected_rows (GtkTreeSelection   *selection,
   do
     {
       if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED))
-	list = g_list_append (list, gtk_tree_path_copy (path));
+	list = g_list_prepend (list, gtk_tree_path_copy (path));
 
       if (node->children)
         {
@@ -506,7 +507,8 @@ gtk_tree_selection_get_selected_rows (GtkTreeSelection   *selection,
 		  if (!tree)
 		    {
 		      gtk_tree_path_free (path);
-		      return list;
+
+		      goto done; 
 		    }
 
 		  gtk_tree_path_up (path);
@@ -519,7 +521,8 @@ gtk_tree_selection_get_selected_rows (GtkTreeSelection   *selection,
 
   gtk_tree_path_free (path);
 
-  return list;
+ done:
+  return g_list_reverse (list);
 }
 
 static void
